@@ -3,7 +3,9 @@ use strict; use warnings;
 
 package B::DeparseTree::Common;
 
-use B qw(class opnumber OPpLVAL_INTRO OPf_SPECIAL OPf_KIDS SVf_ROK
+use B qw(class
+         main_cv main_root main_start
+         opnumber OPpLVAL_INTRO OPf_SPECIAL OPf_KIDS SVf_ROK
          SVf_POK CVf_LVALUE CVf_METHOD
          OPpTARGET_MY svref_2object perlstring);
 use Carp;
@@ -18,15 +20,16 @@ $VERSION = '1.1.0';
             %rev_feature declare_hinthash declare_warnings %ignored_hints
             _features_from_bundle ambiant_pragmas maybe_qualify
             %globalnames gv_name is_scope logop maybe_targmy is_state
-            POSTFIX baseop pfixop deparse_sub deparse_subname next_todo
+            POSTFIX baseop mapop pfixop
+            deparse_sub deparse_subname next_todo
             );
 
 BEGIN {
     # List version-specific constants here.
     # Easiest way to keep this code portable between version looks to
     # be to fake up a dummy constant that will never actually be true.
-    foreach (qw(OPpSORT_INPLACE OPpSORT_DESCEND OPpITER_REVERSED OPpCONST_NOVER
-		OPpPAD_STATE PMf_SKIPWHITE RXf_SKIPWHITE
+    foreach (qw(OPpSORT_INPLACE OPpSORT_DESCEND OPpITER_REVERSED
+                OPpCONST_NOVER OPpPAD_STATE PMf_SKIPWHITE RXf_SKIPWHITE
 		RXf_PMf_CHARSET RXf_PMf_KEEPCOPY
 		CVf_LOCKED OPpREVERSE_INPLACE OPpSUBSTR_REPL_FIRST
 		PMf_NONDESTRUCT OPpCONST_ARYBASE OPpEVAL_BYTES)) {
@@ -128,23 +131,32 @@ sub init($)
     delete $self->{'subs_declared'};
 }
 
-sub coderef2text {
+sub main2info($)
+{
     my $self = shift;
-    my $func = shift;
+    $self->{'curcv'} = B::main_cv;
+    $self->pessimize(B::main_root, B::main_start);
+    return $self->deparse_root(B::main_root);
+}
+
+sub coderef2info($$)
+{
+    my ($self, $coderef) = @_;
+    croak "Usage: ->coderef2info(CODEREF)" unless UNIVERSAL::isa($coderef, "CODE");
+    $self->init();
+    return $self->deparse_sub(svref_2object($coderef));
+}
+
+sub coderef2text($$)
+{
+    my ($self, $func) = @_;
     croak "Usage: ->coderef2text(CODEREF)" unless UNIVERSAL::isa($func, "CODE");
 
     $self->init();
-    my $info = $self->coderef2list($func);
+    my $info = $self->coderef2info($func);
     return $self->indent($info->{text});
 }
 
-sub coderef2list {
-    my ($self, $coderef) = @_;
-    croak "Usage: ->coderef2list(CODEREF)" unless UNIVERSAL::isa($coderef, "CODE");
-    $self->init();
-    return $self->deparse_sub(svref_2object($coderef));
-}
-
 sub print_protos($)
 {
     my $self = shift;
@@ -175,6 +187,12 @@ sub style_opts($$)
     }
 }
 
+sub null
+{
+    my $op = shift;
+    return class($op) eq "NULL";
+}
+
 # This is a special case of scopeop and lineseq, for the case of the
 # main_root.
 sub deparse_root {
@@ -202,7 +220,17 @@ sub deparse_root {
 	$info->{parent} = $$parent if $parent;
 	push @$exprs, $info;
     };
-    return $self->walk_lineseq($op, \@ops, $fn);
+    my $info = $self->walk_lineseq($op, \@ops, $fn);
+    my @other_ops;
+    if (exists $info->{other_ops}) {
+	@other_ops = @{$info->other_ops};
+	push @other_ops, $op->first;
+    } else {
+	@other_ops = ($op->first);
+    }
+    $info->{other_ops} = \@other_ops;
+    return $info;
+
 }
 
 sub is_state {
@@ -263,12 +291,6 @@ sub info_from_text($$$)
     return info_from_list([$text], '', $type, $opts)
 }
 
-sub null
-{
-    my $op = shift;
-    return class($op) eq "NULL";
-}
-
 sub walk_lineseq
 {
     my ($self, $op, $kids, $callback) = @_;
@@ -522,6 +544,131 @@ sub pessimize {
     $self->_pessimize_walk_exe($start, \%visited);
 }
 
+sub stash_subs {
+    my ($self, $pack, $seen) = @_;
+    my (@ret, $stash);
+    if (!defined $pack) {
+	$pack = '';
+	$stash = \%::;
+    }
+    else {
+	$pack =~ s/(::)?$/::/;
+	no strict 'refs';
+	$stash = \%{"main::$pack"};
+    }
+    return
+	if ($seen ||= {})->{
+	    $INC{"overload.pm"} ? overload::StrVal($stash) : $stash
+	   }++;
+    my %stash = svref_2object($stash)->ARRAY;
+    while (my ($key, $val) = each %stash) {
+	my $class = class($val);
+	if ($class eq "PV") {
+	    # Just a prototype. As an ugly but fairly effective way
+	    # to find out if it belongs here is to see if the AUTOLOAD
+	    # (if any) for the stash was defined in one of our files.
+	    my $A = $stash{"AUTOLOAD"};
+	    if (defined ($A) && class($A) eq "GV" && defined($A->CV)
+		&& class($A->CV) eq "CV") {
+		my $AF = $A->FILE;
+		next unless $AF eq $0 || exists $self->{'files'}{$AF};
+	    }
+	    push @{$self->{'protos_todo'}}, [$pack . $key, $val->PV];
+	} elsif ($class eq "IV" && !($val->FLAGS & SVf_ROK)) {
+	    # Just a name. As above.
+	    # But skip proxy constant subroutines, as some form of perl-space
+	    # visible code must have created them, be it a use statement, or
+	    # some direct symbol-table manipulation code that we will Deparse
+	    my $A = $stash{"AUTOLOAD"};
+	    if (defined ($A) && class($A) eq "GV" && defined($A->CV)
+		&& class($A->CV) eq "CV") {
+		my $AF = $A->FILE;
+		next unless $AF eq $0 || exists $self->{'files'}{$AF};
+	    }
+	    push @{$self->{'protos_todo'}}, [$pack . $key, undef];
+	} elsif ($class eq "GV") {
+	    if (class(my $cv = $val->CV) ne "SPECIAL") {
+		next if $self->{'subs_done'}{$$val}++;
+		next if $$val != ${$cv->GV};   # Ignore imposters
+		$self->todo($cv, 0);
+	    }
+	    if (class(my $cv = $val->FORM) ne "SPECIAL") {
+		next if $self->{'forms_done'}{$$val}++;
+		next if $$val != ${$cv->GV};   # Ignore imposters
+		$self->todo($cv, 1);
+	    }
+	    if (class($val->HV) ne "SPECIAL" && $key =~ /::$/) {
+		$self->stash_subs($pack . $key, $seen);
+	    }
+	}
+    }
+}
+
+sub compile {
+    my(@args) = @_;
+    return sub {
+	my $self = B::DeparseTree->new(@args);
+	# First deparse command-line args
+	if (defined $^I) { # deparse -i
+	    print q(BEGIN { $^I = ).perlstring($^I).qq(; }\n);
+	}
+	if ($^W) { # deparse -w
+	    print qq(BEGIN { \$^W = $^W; }\n);
+	}
+	if ($/ ne "\n" or defined $O::savebackslash) { # deparse -l and -0
+	    my $fs = perlstring($/) || 'undef';
+	    my $bs = perlstring($O::savebackslash) || 'undef';
+	    print qq(BEGIN { \$/ = $fs; \$\\ = $bs; }\n);
+	}
+	my @BEGINs  = B::begin_av->isa("B::AV") ? B::begin_av->ARRAY : ();
+	my @UNITCHECKs = B::unitcheck_av->isa("B::AV")
+	    ? B::unitcheck_av->ARRAY
+	    : ();
+	my @CHECKs  = B::check_av->isa("B::AV") ? B::check_av->ARRAY : ();
+	my @INITs   = B::init_av->isa("B::AV") ? B::init_av->ARRAY : ();
+	my @ENDs    = B::end_av->isa("B::AV") ? B::end_av->ARRAY : ();
+	for my $block (@BEGINs, @UNITCHECKs, @CHECKs, @INITs, @ENDs) {
+	    $self->todo($block, 0);
+	}
+	$self->stash_subs();
+	local($SIG{"__DIE__"}) =
+	  sub {
+	      if ($self->{'curcop'}) {
+		  my $cop = $self->{'curcop'};
+		  my($line, $file) = ($cop->line, $cop->file);
+		  print STDERR "While deparsing $file near line $line,\n";
+	      }
+	    };
+	$self->{'curcv'} = main_cv;
+	$self->{'curcvlex'} = undef;
+	print $self->print_protos;
+	@{$self->{'subs_todo'}} =
+	  sort {$a->[0] <=> $b->[0]} @{$self->{'subs_todo'}};
+	my $root = main_root;
+	local $B::overlay = {};
+	unless (null $root) {
+	    $self->pessimize($root, main_start);
+	    my $deparsed = $self->deparse_root($root);
+	    print $self->indent_info($deparsed), "\n";
+	}
+	my @texts;
+	while (scalar(@{$self->{'subs_todo'}})) {
+	    push @texts, $self->next_todo;
+	}
+	print $self->indent(join("", @texts)), "\n" if @texts;
+
+	# Print __DATA__ section, if necessary
+	no strict 'refs';
+	my $laststash = defined $self->{'curcop'}
+	    ? $self->{'curcop'}->stash->NAME : $self->{'curstash'};
+	if (defined *{$laststash."::DATA"}{IO}) {
+	    print "package $laststash;\n"
+		unless $laststash eq $self->{'curstash'};
+	    print "__DATA__\n";
+	    print readline(*{$laststash."::DATA"});
+	}
+    }
+}
 
 # This method is the inner loop, so try to keep it simple
 sub deparse($$$$)
@@ -1030,4 +1177,47 @@ sub pfixop
 			  {maybe_parens => [$self, $cx, $prec]});
 }
 
+sub mapop
+{
+    my($self, $op, $cx, $name) = @_;
+    my $kid = $op->first; # this is the (map|grep)start
+
+    my @other_ops = ($kid, $kid->first);
+    $kid = $kid->first->sibling; # skip a pushmark
+
+    my $code = $kid->first; # skip a null
+
+    my $code_info;
+
+    my @block_texts = ();
+    my @exprs_texts = ();
+    if (is_scope $code) {
+	$code_info = $self->deparse($code, 0, $op);
+	(my $text = $code_info->{text})=~ s/^\n//;  # remove first \n in block.
+	@block_texts = ('{', $text, '}');
+    } else {
+	$code_info = $self->deparse($code, 24, $op);
+	@exprs_texts = ($code_info->{text});
+    }
+    my @body = ($code_info);
+
+    push @other_ops, $kid;
+    $kid = $kid->sibling;
+    my($expr, @exprs);
+    for (; !null($kid); $kid = $kid->sibling) {
+	$expr = $self->deparse($kid, 6, $op);
+	push @exprs, $expr if defined $expr;
+    }
+    push @body, @exprs;
+    push @exprs_texts, map $_->{text}, @exprs;
+    my $opts = {
+	body => \@body,
+	other_ops => \@other_ops,
+    };
+    my $params = join(', ', @exprs_texts);
+    $params = join(" ", @block_texts) . ' ' . $params if @block_texts;
+    my @texts = $self->maybe_parens_func($name, $params, $cx, 5);
+    return info_from_list(\@texts, '', 'mapop', $opts)
+}
+
 1;
