@@ -21,8 +21,9 @@ $VERSION = '1.0.0';
     pp_eprotoent pp_epwent pp_eservent
     pp_fork pp_getlogin pp_ggrent
     pp_ghostent pp_gnetent pp_gprotoent
-    pp_gpwent pp_gservent pp_sgrent
-    pp_spwent pp_tms pp_wantarray
+    pp_gpwent pp_grepstart pp_gservent
+    pp_mapstart
+    pp_sgrent pp_spwent pp_tms pp_wantarray
 
     pp_leave pp_lineseq pp_scope
 
@@ -30,6 +31,8 @@ $VERSION = '1.0.0';
 
     pp_and pp_or pp_dor pp_xor
 
+    pp_mapwhile pp_grepwhile
+
     pp_complement
     pp_getppid
     pp_postdec
@@ -64,7 +67,9 @@ sub pp_ghostent { baseop(@_, "gethostent") }
 sub pp_gnetent { baseop(@_, "getnetent") }
 sub pp_gprotoent { baseop(@_, "getprotoent") }
 sub pp_gpwent { baseop(@_, "getpwent") }
+sub pp_grepstart { baseop(@_, "grep") }
 sub pp_gservent { baseop(@_, "getservent") }
+sub pp_mapstart { baseop(@_, "map") }
 sub pp_sgrent { baseop(@_, "setgrent") }
 sub pp_spwent { baseop(@_, "setpwent") }
 sub pp_tms { baseop(@_, "times") }
@@ -178,6 +183,9 @@ sub pp_dor { logop(@_, "//", 10) }
 # old versions of opcode.pl). Syntax is what matters here.
 sub pp_xor { logop(@_, "xor", 2, "",   0,  "") }
 
+sub pp_mapwhile { mapop(@_, "map") }
+sub pp_grepwhile { mapop(@_, "grep") }
+
 sub pp_complement { maybe_targmy(@_, \&pfixop, "~", 21) }
 sub pp_getppid { maybe_targmy(@_, \&baseop, "getppid") }
 sub pp_postdec { maybe_targmy(@_, \&pfixop, "--", 23, POSTFIX) }
@@ -190,6 +198,4 @@ sub pp_predec { pfixop(@_, "--", 23) }
 sub pp_i_preinc { pfixop(@_, "++", 23) }
 sub pp_i_predec { pfixop(@_, "--", 23) }
 
-
-
 1;
